!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
!   r3b :    The Restricted 3-Body Problem
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 

 SUBROUTINE GETRHS(NDIM,U,PAR,F)
!---------- ------

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
 
! Define the RHS of the CR3BP

   DOUBLE PRECISION x,y,z,xp,yp,zp,mu,logmu,dE,dM,mc,dE3,dM3
   DOUBLE PRECISION beta,alpha,delta,rsx,rsy,rsz,r2
   DOUBLE PRECISION sa,sd,ca,cd,Nx,Ny,Nz,dotrsn,dotrsn2

     x  = U(1)        
     y  = U(2) 
     z  = U(3) 
     xp = U(4) 
     yp = U(5) 
     zp = U(6) 

! For numerical robustness, log(mu) is used as the continuation 
! parameter rather than the mass-ratio mu itself.

     logmu = PAR(15)
     mu    = DEXP(logmu)
     alpha = PAR(16)
     beta  = PAR(17)
     delta = PAR(18)

     dE  = SQRT((x+mu)**2 + y**2 + z**2)
     dM  = SQRT( (x-1+mu)**2 + y**2 + z**2 )
     mc  = 1 - mu
     dE3 = 1/dE**3
     dM3 = 1/dM**3

     rsx = (x+mu)/dE
     rsy = y/dE
     rsz = z/dE
     r2  = SQRT((x+mu)**2 + y**2)

     sa = SIN(alpha)
     sd = SIN(delta)
     ca = COS(alpha)
     cd = COS(delta)

     Nx = (ca*(x+mu) - sa*y)*(cd - sd*z/r2)/dE
     Ny = (ca*y - sa*(x+mu))*(cd - sd*z/r2)/dE
     Nz = (cd*z + sd*r2)/dE

     dotrsn = (rsx*Nx + rsy*Ny + rsz*Nz)
     dotrsn2 = dotrsn**2

     F(1)= xp
     F(2)= yp
     F(3)= zp
     F(4)= 2*yp + x - mc*dE3*(x+mu)  - mu*dM3*(x-1+mu) + beta*mc*dE3*dotrsn2*Nx 
     F(5)=-2*xp + y - mc*dE3*y       - mu*dM3*y        + beta*mc*dE3*dotrsn2*Ny 
     F(6)=          - mc*dE3*z       - mu*dM3*z        + beta*mc*dE3*dotrsn2*Nz 

 END SUBROUTINE GETRHS
!----------------------------------------------------------------------

 SUBROUTINE GETJAC(NDIM,U,PAR,DFDU)
!---------- ------

   IMPLICIT NONE
   INTEGER, INTENT(IN) :: NDIM
   DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
   DOUBLE PRECISION, INTENT(OUT) :: DFDU(NDIM,*)
   DOUBLE PRECISION x,y,z,xp,yp,zp,mu,logmu,dE,dM,mc,dE3,dM3,dE5,dM5
   DOUBLE PRECISION beta,alpha,delta,ca,cd,sa,sd,dE2,dE2x,dE2y,dE2z
   DOUBLE PRECISION dEx,dEy,dEz,cx,cy,cz,dx,dy,dz,ex,ey,ez,vx,vy,vz
   DOUBLE PRECISION rsx,rsy,rsz,r2,r23,rsxx,rsxy,rsxz,rsyx,rsyy,rsyz
   DOUBLE PRECISION rszx,rszy,rszz,Nx,Ny,Nz,dotrsn,dotrsn2 
   DOUBLE PRECISION Nxx,Nxy,Nxz,Nyx,Nyy,Nyz,Nzx,Nzy,Nzz
   DOUBLE PRECISION dotx,doty,dotz,Bcx,Bcy,Bcz,Bdx,Bdy,Bdz,Bex,Bey,Bez

   DOUBLE PRECISION vxp,vyp,vzp,dMx,dMy,dMz,dEmu,dMmu
 
! Define the Jacobian:

     x  = U(1)
     y  = U(2)
     z  = U(3)
     xp = U(4)
     yp = U(5)
     zp = U(6)

     logmu = PAR(15)
     mu    = DEXP(logmu)
     alpha = PAR(16)
     beta  = PAR(17)
     delta = PAR(18)

     dE  = SQRT((x+mu)**2 + y**2 + z**2)
     dM  = SQRT( (x-1+mu)**2 + y**2 + z**2 )
     mc  = 1 - mu
     dE3 = 1/dE**3
     dM3 = 1/dM**3
     dE2 = 1/dE**2

     dE5 = 1/dE**5
     dM5 = 1/dM**5

     dEx = -3*(x+mu)*dE5
     dEy = -3*y*dE5
     dEz = -3*z*dE5

     dMx = -3*(x-1+mu)*dM5
     dMy = -3*y*dM5
     dMz = -3*z*dM5

     dE2x = -2*(x+mu)/(dE**4)
     dE2y = -2*y/(dE**4)
     dE2z = -2*z/(dE**4)

     sa = SIN(alpha)
     sd = SIN(delta)
     ca = COS(alpha)
     cd = COS(delta)

     rsx = (x+mu)/dE
     rsy = y/dE
     rsz = z/dE
     r2  = SQRT((x+mu)**2 + y**2)
     r23 = r2**3

     rsxx = (1/dE) - ((x+mu)**2)*dE3
     rsxy = -(x+mu)*y*dE3
     rsxz = -(x+mu)*z*dE3

     rsyx = -(x+mu)*y*dE3
     rsyy = (1/dE) - (y**2)*dE3
     rsyz = -y*z*dE3

     rszx = -(x+mu)*z*dE3
     rszy = -y*z*dE3
     rszz = (1/dE) - (z**2)*dE3

     Nx = (ca*(x+mu) - sa*y)*(cd - sd*z/r2)/dE
     Ny = (ca*y - sa*(x+mu))*(cd - sd*z/r2)/dE
     Nz = (cd*z + sd*r2)/dE
     dotrsn = (rsx*Nx + rsy*Ny + rsz*Nz)
     dotrsn2 = dotrsn**2

     Nxx = (ca*cd - rsx*Nx - (ca*sd*z/r2) + (sd*(x+mu)*z/r23)*(ca*(x+mu)-sa*y))/dE
     Nxy = (-sa*cd - rsy*Nx + (sa*sd*z/r2) + (sd*y*z/r23)*(ca*(x+mu)-sa*y))/dE
     Nxz = (-rsz*Nx - (sd/r2)*(ca*(x+mu)-sa*y))/dE

     Nyx = (sa*cd - rsx*Ny - (sa*cd*z/r2) + (sd*(x+mu)*z/r23)*(ca*y+sa*(x+mu)))/dE
     Nyy = (ca*cd - rsy*Ny - (ca*sd*z/r2) + (sd*y*z/r23)*(ca*y+sa*(x+mu)))/dE
     Nyz = (-rsz*Ny - (sd/r2)*(ca*y+sa*(x+mu)))/dE

     Nzx = (x+mu)*(-(Nz/dE) + (sd/r2))/dE
     Nzy = y*(-(Nz/dE) + (sd/r2))/dE
     Nzz = (cd - (z*Nz/dE))/dE
    
     dotx = rsxx*Nx + rsx*Nxx + rsyx*Ny + rsy*Nyx + rszx*Nz + rsz*Nzx
     doty = rsxy*Nx + rsx*Nxy + rsyy*Ny + rsy*Nyy + rszy*Nz + rsz*Nzy
     dotz = rsxz*Nx + rsx*Nxz + rsyz*Ny + rsy*Nyz + rszz*Nz + rsz*Nzz

     Bcx = beta*mc*dotrsn*(dE2x*dotrsn*Nx + 2*dE2*dotx*Nx + dE2*dotrsn*Nxx)
     Bcy = beta*mc*dotrsn*(dE2y*dotrsn*Nx + 2*dE2*doty*Nx + dE2*dotrsn*Nxy)
     Bcz = beta*mc*dotrsn*(dE2z*dotrsn*Nx + 2*dE2*dotz*Nx + dE2*dotrsn*Nxz)

     Bdx = beta*mc*dotrsn*(dE2x*dotrsn*Ny + 2*dE2*dotx*Ny + dE2*dotrsn*Nyx)
     Bdy = beta*mc*dotrsn*(dE2y*dotrsn*Ny + 2*dE2*doty*Ny + dE2*dotrsn*Nyy)
     Bdz = beta*mc*dotrsn*(dE2z*dotrsn*Ny + 2*dE2*dotz*Ny + dE2*dotrsn*Nyz)

     Bex = beta*mc*dotrsn*(dE2x*dotrsn*Nz + 2*dE2*dotx*Nz + dE2*dotrsn*Nzx)
     Bey = beta*mc*dotrsn*(dE2y*dotrsn*Nz + 2*dE2*doty*Nz + dE2*dotrsn*Nzy)
     Bez = beta*mc*dotrsn*(dE2z*dotrsn*Nz + 2*dE2*dotz*Nz + dE2*dotrsn*Nzz)

     cx = 1 - mc*dE3 - mc*(x+mu)*dEx - mu*dM3 - mu*(x-1+mu)*dMx + Bcx
     cy = -mc*(x+mu)*dEy - mu*(x-1+mu)*dMy + Bcy
     cz = -mc*(x+mu)*dEz - mu*(x-1+mu)*dMz + Bcz

     dx = -mc*y*dEx - mu*y*dMx + Bdx
     dy = 1 - mc*dE3 - mc*y*dEy - mu*dM3 - mu*y*dMy + Bdy
     dz = -mc*y*dEz - mu*y*dMz + Bdz

     ex = -mc*z*dEx - mu*z*dMx + Bex
     ey = -mc*z*dEy - mu*z*dMy + Bey
     ez = -mc*dE3 - mc*z*dEz - mu*dM3 - mu*z*dMz + Bez

     DFDU(1,1) = 0.d0
     DFDU(1,2) = 0.d0
     DFDU(1,3) = 0.d0
     DFDU(1,4) = 1.d0
     DFDU(1,5) = 0.d0
     DFDU(1,6) = 0.d0

     DFDU(2,1) = 0.d0
     DFDU(2,2) = 0.d0
     DFDU(2,3) = 0.d0
     DFDU(2,4) = 0.d0
     DFDU(2,5) = 1.d0
     DFDU(2,6) = 0.d0

     DFDU(3,1) = 0.d0
     DFDU(3,2) = 0.d0
     DFDU(3,3) = 0.d0
     DFDU(3,4) = 0.d0
     DFDU(3,5) = 0.d0
     DFDU(3,6) = 1.d0

     DFDU(4,1) = cx
     DFDU(4,2) = cy
     DFDU(4,3) = cz
     DFDU(4,4) = 0.d0
     DFDU(4,5) = 2.d0
     DFDU(4,6) = 0.d0

     DFDU(5,1) = dx
     DFDU(5,2) = dy
     DFDU(5,3) = dz
     DFDU(5,4) =-2.d0
     DFDU(5,5) = 0.d0
     DFDU(5,6) = 0.d0

     DFDU(6,1) = ex
     DFDU(6,2) = ey
     DFDU(6,3) = ez
     DFDU(6,4) = 0.d0
     DFDU(6,5) = 0.d0
     DFDU(6,6) = 0.d0

 END SUBROUTINE GETJAC
!---------------------------------------------------------------------- 
!---------------------------------------------------------------------- 
